-- Common imports.
import "Turbine";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Gameplay";
import "Turbine.Animation";
import "Turbine.UI.Extensions";

import "ROCKsPlugins.Moralbar.VindarPatch"


-- comment the load
Turbine.Shell.WriteLine("Moralebar v1.1.4 by ROCKsPlugins")

-- get resolution
screenWidth, screenHeight = Turbine.UI.Display:GetSize();

-- initial values
startTime = 0;
tarstartTime = 0;

settingsDataScope = Turbine.DataScope.Character;
settingsFilename = "Moralebar_Settings";

DEFAULT_SETTINGS = {
    ["WINDOWS"] = {
        ["WIDTH"] = 500;
        ["HEIGHT"] = 40;
        ["TARWIDTH"] = 1000;
        ["TARHEIGHT"] = 20;
        ["POWWIDTH"] = 1;
        ["POWHEIGHT"] = 8;
        ["OPACITY"] = 0.4;
        ["HEIGHTFROMGROUND"] = 200;
        ["HEIGHTFROMGROUNDTARGET"] = 700;
        ["ORIGINAL_MORALEBAR"] = 1;
        ["ORIGINAL_TARGETBAR"] = 1;
        ["PLUGIN_TARGETBAR"] = 1;
        ["ORIGINAL_TARGETBAR"] = 1;
        ["ORIGINAL_MORALEBAR"] = 3;
        ["POWERBAR"] = 1;
    }
}

SETTINGS = {}

function LoadSettings()
    local loadedSettings = PatchDataLoad(
        settingsDataScope, 
        settingsFilename);

    -- did load something good?
    if (type(loadedSettings) == 'table') then
        -- Yes, usw what loaded
        SETTINGS = loadedSettings;
    else
        -- No, start with the default values, they're OK.
        SETTINGS = DEFAULT_SETTINGS;
    end
end

function SaveSettings()
    local screenWidth, screenHeight = Turbine.UI.Display:GetSize();
    SETTINGS.WINDOWS.HEIGHTFROMGROUNDTARGET = screenHeight - targetWindow:GetTop() - SETTINGS.WINDOWS.TARHEIGHT
  
    SETTINGS.WINDOWS.HEIGHTFROMGROUND = screenHeight - window:GetTop() - SETTINGS.WINDOWS.HEIGHT

    PatchDataSave(
        settingsDataScope,
        settingsFilename,
        SETTINGS);
    Turbine.Shell.WriteLine("Moralebar settings saved.")
end

function RegisterForUnload()
    Turbine.Plugin.Unload = function (sender, args)
        SaveSettings();
    end
end

-- Background for label
Background = "ROCKsPlugins/Moralbar/Resources/Verlauf.tga"
Background2 = "ROCKsPlugins/Moralbar/Resources/Verlauf2.tga"
Background3 = "ROCKsPlugins/Moralbar/Resources/Verlauf3.tga"
Background_t = "ROCKsPlugins/Moralbar/Resources/Verlauf_2.tga"
Background_t2 = "ROCKsPlugins/Moralbar/Resources/Verlauf2_2.tga"
Background_t3 = "ROCKsPlugins/Moralbar/Resources/Verlauf3_2.tga"


--------------------------------------------------------------------
-- player ui


function playerUI()
    -- actor vitalbar
    window = Turbine.UI.Window();
    window:SetPosition(screenWidth/2 - SETTINGS.WINDOWS.WIDTH/2, screenHeight - SETTINGS.WINDOWS.HEIGHT - SETTINGS.WINDOWS.HEIGHTFROMGROUND);
    window:SetSize(SETTINGS.WINDOWS.WIDTH, SETTINGS.WINDOWS.HEIGHT);
    window:SetVisible(true);
    window:SetBackColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 0, 0, 0));
    window:SetZOrder(-1);
    window:SetBackground(Background);
    window:SetBackColorBlendMode( Turbine.UI.BlendMode.Multiply );

    -- actor powerbar
    if (SETTINGS.WINDOWS.POWERBAR == 1) then
    powerwindow = Turbine.UI.Window();
    powerwindow:SetParent(window);
    powerwindow:SetPosition((1-1/SETTINGS.WINDOWS.POWWIDTH)*SETTINGS.WINDOWS.WIDTH, (1-1/SETTINGS.WINDOWS.POWHEIGHT)*SETTINGS.WINDOWS.HEIGHT);
    powerwindow:SetSize(SETTINGS.WINDOWS.WIDTH/SETTINGS.WINDOWS.POWWIDTH, SETTINGS.WINDOWS.HEIGHT/SETTINGS.WINDOWS.POWHEIGHT);
    powerwindow:SetVisible(true);
    powerwindow:SetBackColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 0, 0, 1));
    powerwindow:SetZOrder(0);
    powerwindow:SetBackground(Background);
    powerwindow:SetBackColorBlendMode( Turbine.UI.BlendMode.Multiply );

    -- power change label
    powerlabel4 = Turbine.UI.Label()
    powerlabel4:SetParent(powerwindow);
    powerlabel4:SetVisible(false);
    powerlabel4:SetSize(SETTINGS.WINDOWS.WIDTH/SETTINGS.WINDOWS.POWWIDTH, SETTINGS.WINDOWS.HEIGHT/SETTINGS.WINDOWS.POWHEIGHT);
    powerlabel4:SetBackColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 0, 0, 0));
    powerlabel4:SetZOrder(1);
    powerlabel4:SetBackground(Background2);
    powerlabel4:SetBackColorBlendMode( Turbine.UI.BlendMode.Multiply );
    end

    -- morale label
    label1 = Turbine.UI.Label();
    label1:SetParent(window);
    label1:SetSize(SETTINGS.WINDOWS.WIDTH, SETTINGS.WINDOWS.HEIGHT);
    label1:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
    label1:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
    label1:SetOutlineColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 0, 0, 0));
    label1:SetFontStyle( Turbine.UI.FontStyle.Outline )
    label1:SetFont( Turbine.UI.Lotro.Font.Verdana20 )

    -- percent label
    label2 = Turbine.UI.Label()
    label2:SetParent(window);
    label2:SetSize(SETTINGS.WINDOWS.WIDTH, SETTINGS.WINDOWS.HEIGHT);
    label2:SetText("100 % ");
    label2:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleRight );
    label2:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
    label2:SetOutlineColor(Turbine.UI.Color(1, 0, 0, 0));

    -- + label
    label3 = Turbine.UI.Label()
    label3:SetParent(window);
    label3:SetSize(SETTINGS.WINDOWS.WIDTH, SETTINGS.WINDOWS.HEIGHT);
    label3:SetText(" +"); 
    label3:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
    label3:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
    label3:SetOutlineColor(Turbine.UI.Color(1, 0, 0, 0));
    label3:SetFontStyle( Turbine.UI.FontStyle.Outline )
    label3:SetFont( Turbine.UI.Lotro.Font.Verdana16 )

    -- morale change label
    label4 = Turbine.UI.Label()
    label4:SetParent(window);
    label4:SetVisible(false);
    label4:SetSize(SETTINGS.WINDOWS.WIDTH, SETTINGS.WINDOWS.HEIGHT);
    label4:SetBackColor(Turbine.UI.Color(1, 1, 0, 0));
    label4:SetZOrder(-1);
    label4:SetBackground(Background2);
    label4:SetBackColorBlendMode( Turbine.UI.BlendMode.Multiply );

    -- hit label, delta between two hits
    widthLast = SETTINGS.WINDOWS.WIDTH;

    label5 = Turbine.UI.Label()
    label5:SetParent(window);
    label5:SetVisible(false);
    label5:SetSize(widthLast, SETTINGS.WINDOWS.HEIGHT);
    label5:SetBackColor(Turbine.UI.Color(1, 1, 1, 1));
    label5:SetZOrder(-2);
end


----------------------------------------------------
-- player functions


function playerMethods()
    -- load LocalPlayer
    player = Turbine.Gameplay.LocalPlayer.GetInstance();
    local maxMorale = math.floor(player:GetMaxMorale());
    label1:SetText(maxMorale);

    player.InCombatChanged = function()
        if player:IsInCombat() == true then
            -- set label visibility
            label4:SetVisible(true); -- visible
            label5:SetVisible(true); -- visible
            -- opaticity change from text labels
            label1:SetForeColor(Turbine.UI.Color(1, 1, 1, 1));
            label2:SetForeColor(Turbine.UI.Color(1, 1, 1, 1));
            label3:SetForeColor(Turbine.UI.Color(1, 1, 1, 1));
            window:SetBackColor(Turbine.UI.Color(0.8, 0, 0, 0));
            window:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend );
            if (SETTINGS.WINDOWS.POWERBAR == 1) then
                powerlabel4:SetVisible(true);
                powerwindow:SetBackColor(Turbine.UI.Color(0.8, 0, 0, 0));
                powerwindow:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend );
                powerlabel4:SetBackColor(Turbine.UI.Color(1, 0, 0, 1));
            end
        else      
            -- set label visibility
            if (SETTINGS.WINDOWS.POWERBAR == 1) then
                powerlabel4:SetVisible(false);
                powerwindow:SetBackColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 0, 0, 0));
                powerwindow:SetBackColorBlendMode( Turbine.UI.BlendMode.Multiply );
                powerlabel4:SetBackColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 0, 0, 0));
            end
            label4:SetVisible(false); -- visible
            label5:SetVisible(false); -- visible
            -- opaticity change from text labels
            label1:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
            label2:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
            label3:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
            window:SetBackColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 0, 0, 0));
            window:SetBackColorBlendMode( Turbine.UI.BlendMode.Multiply );
            label4:SetSize(SETTINGS.WINDOWS.WIDTH, SETTINGS.WINDOWS.HEIGHT);
        end
    end

    -- routine by morale change
    player.MoraleChanged = function (sender, args)
        -- get actualy morale
        local actuMorale = math.floor(player:GetMorale());
        local maxMorale = math.floor(player:GetMaxMorale());
        -- Morale in percent
        local percentMorale = math.floor((actuMorale / maxMorale * 100));
        local actuMoraleStr = tostring(actuMorale);
        local maxMoraleStr = tostring(maxMorale);
        local percentMoraleStr = tostring(percentMorale);

        -- display Morale
        if percentMorale == 100 then
            label1:SetText(maxMorale);
            
        else
            label1:SetText(actuMorale .. " / " .. maxMorale);
                    
            -- set remain morale label
            local widthChange = SETTINGS.WINDOWS.WIDTH * percentMorale / 100;
            label4:SetSize(widthChange, SETTINGS.WINDOWS.HEIGHT);

            -- SETTINGS.WINDOWS.WIDTH change for morale between two hits
            local widthDelta = widthLast - widthChange;
            widthLast = widthLast - widthDelta;
            label5:SetSize((widthLast + widthDelta), SETTINGS.WINDOWS.HEIGHT);
            label5:SetWantsUpdates(true);
            

            label5.Update = function (sender, args)
                -- delay realize, smooth delta morale reduce
                local currentTime = math.floor(Turbine.Engine.GetGameTime()); -- in seconds
                if (startTime > currentTime) then
                    -- Reduce the hit label SETTINGS.WINDOWS.WIDTH           
                    widthDelta = widthDelta - 1;
                    label5:SetSize((widthLast + widthDelta), SETTINGS.WINDOWS.HEIGHT);
                    if widthDelta == 0 then 
                        label5:SetWantsUpdates(false);
                    end
                end                
                startTime = currentTime + 1;       
            end
        end
        --display percent Morale
        label2:SetText(percentMorale .. " % ");
    end 


    -- routine by max morale change
    player.MaxMoraleChanged = function (sender, args)
        -- get actualy morale
        --local actuMorale = math.floor(player:GetMorale());
        local maxMorale = math.floor(player:GetMaxMorale());
        -- Morale in percent
        --local percentMorale = math.floor((actuMorale / maxMorale * 100));
        --local actuMoraleStr = tostring(actuMorale);
        local maxMoraleStr = tostring(maxMorale);
        --local percentMoraleStr = tostring(percentMorale);
    
        if player:IsInCombat() == false then
            label1:SetText(maxMoraleStr);
        end
    end

    if (SETTINGS.WINDOWS.POWERBAR == 1) then
    local maxPower = math.floor(player:GetMaxPower());

    player.PowerChanged = function()
        local actuPower = math.floor(player:GetPower());
        --Turbine.Shell.WriteLine(actuPower)
        local percentPower = math.floor((actuPower / maxPower * 100));
        --Turbine.Shell.WriteLine(percentPower)
        -- set remain power label
        local powwidthChange = SETTINGS.WINDOWS.WIDTH/SETTINGS.WINDOWS.POWWIDTH * percentPower / 100;
        powerlabel4:SetSize(powwidthChange, SETTINGS.WINDOWS.HEIGHT/SETTINGS.WINDOWS.POWWIDTH);
    end
    end
end


---------------------------------------------------------------------
--target ui


function targetUI()
    -- actor vitalbar
    targetWindow = Turbine.UI.Window();
    targetWindow:SetPosition(screenWidth/2 - SETTINGS.WINDOWS.TARWIDTH/2, screenHeight - SETTINGS.WINDOWS.TARHEIGHT - SETTINGS.WINDOWS.HEIGHTFROMGROUNDTARGET);
    targetWindow:SetSize(SETTINGS.WINDOWS.TARWIDTH, SETTINGS.WINDOWS.TARHEIGHT);
    targetWindow:SetVisible(false);
    targetWindow:SetBackColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 0, 0, 0));
    targetWindow:SetZOrder(-1);
    targetWindow:SetBackground(Background_t);
    targetWindow:SetBackColorBlendMode( Turbine.UI.BlendMode.Normal );

    -- name label
    labelTarget0 = Turbine.UI.Label();
    labelTarget0:SetParent(targetWindow);
    labelTarget0:SetSize(SETTINGS.WINDOWS.TARWIDTH, SETTINGS.WINDOWS.TARHEIGHT);
    labelTarget0:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
    labelTarget0:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
    labelTarget0:SetOutlineColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 0, 0, 0));
    labelTarget0:SetFontStyle( Turbine.UI.FontStyle.Outline )
    labelTarget0:SetFont( Turbine.UI.Lotro.Font.Verdana20 )

    -- morale label
    labelTarget1 = Turbine.UI.Label();
    labelTarget1:SetParent(targetWindow);
    labelTarget1:SetSize(SETTINGS.WINDOWS.TARWIDTH, SETTINGS.WINDOWS.TARHEIGHT);
    labelTarget1:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
    labelTarget1:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
    labelTarget1:SetOutlineColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 0, 0, 0));
    labelTarget1:SetFontStyle( Turbine.UI.FontStyle.Outline )
    labelTarget1:SetFont( Turbine.UI.Lotro.Font.Verdana20 )

    -- percent label
    labelTarget2 = Turbine.UI.Label()
    labelTarget2:SetParent(targetWindow);
    labelTarget2:SetSize(SETTINGS.WINDOWS.TARWIDTH, SETTINGS.WINDOWS.TARHEIGHT);
    labelTarget2:SetText("100 % ");
    labelTarget2:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleRight );
    labelTarget2:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
    labelTarget2:SetOutlineColor(Turbine.UI.Color(1, 0, 0, 0));

    -- + label
    --labelTarget3 = Turbine.UI.Label()
    --labelTarget3:SetParent(targetWindow);
    --labelTarget3:SetSize(SETTINGS.WINDOWS.TARWIDTH, SETTINGS.WINDOWS.TARHEIGHT);
    --labelTarget3:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
    --labelTarget3:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
    --labelTarget3:SetOutlineColor(Turbine.UI.Color(1, 0, 0, 0));
    --labelTarget3:SetFontStyle( Turbine.UI.FontStyle.Outline )
    --labelTarget3:SetFont( Turbine.UI.Lotro.Font.Verdana16 )

    -- morale change label
    labelTarget4 = Turbine.UI.Label()
    labelTarget4:SetParent(targetWindow);
    labelTarget4:SetVisible(false);
    labelTarget4:SetSize(SETTINGS.WINDOWS.TARWIDTH, SETTINGS.WINDOWS.TARHEIGHT);
    labelTarget4:SetBackColor(Turbine.UI.Color(1, 1, 0, 0));
    labelTarget4:SetZOrder(-1);
    labelTarget4:SetBackground(Background_t2);
    labelTarget4:SetBackColorBlendMode( Turbine.UI.BlendMode.Multiply );

    -- hit label, delta between two hits
    tarwidthLast = SETTINGS.WINDOWS.TARWIDTH;

    labelTarget5 = Turbine.UI.Label()
    labelTarget5:SetParent(targetWindow);
    labelTarget5:SetVisible(false);
    labelTarget5:SetSize(tarwidthLast, SETTINGS.WINDOWS.TARHEIGHT);
    labelTarget5:SetBackColor(Turbine.UI.Color(1, 1, 1, 1));
    labelTarget5:SetZOrder(-2);

    --Start Print
    labelTarget2:SetText("100 % ");
    --labelTarget3:SetText(" +"); 
end


--------------------------------------------------------
-- target functions

test = Turbine.UI.Window();
function targetMethods()
    target = nil
    
    player.TargetChanged = function()
        labelTarget1:SetWantsUpdates(false);
        if (SETTINGS.WINDOWS.PLUGIN_TARGETBAR == 1) then
            local starttime = Turbine.Engine.GetGameTime()
            local delay = 0.4
            local endtime = starttime + delay
        
            local morale = 0
            target = player:GetTarget()
            if (target == nil) then
                test:SetWantsUpdates(false);
                --Turbine.Shell.WriteLine("false")
    
                targetWindow:SetVisible(false);
                labelTarget1:SetWantsUpdates(false);
            else
                    
                --Turbine.Shell.WriteLine("else")
    
                local targetName = target:GetName()
                -- now test for a function that is defined in the class derived from Entity
                if target.GetPower~=nil then
                    test:SetWantsUpdates(true);
    
                    labelTarget0:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
                    labelTarget2:SetVisible(true);
                    labelTarget1:SetVisible(true);
                    --Turbine.Shell.WriteLine("Power vorhanden")


                    local targetMorale = math.floor(target:GetMorale())
                    local targetMoraleStr = tostring(targetMorale);
                    labelTarget1:SetText(targetMoraleStr);
    
    
                    local targetMaxMorale = math.floor(target:GetMaxMorale())
                    if (targetMaxMorale == 0) then
                        labelTarget0:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
                        labelTarget1:SetVisible(false);
                        labelTarget2:SetVisible(false);
                        -- set label visibility
                        labelTarget4:SetVisible(false); -- visible
                        labelTarget5:SetVisible(false); -- visible
                        -- opaticity change from text labels
                        labelTarget0:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
                        labelTarget1:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
                        labelTarget2:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
                        --labelTarget3:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
                        targetWindow:SetBackColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 0, 0, 0));
                        labelTarget1:SetWantsUpdates(false);
                        --Turbine.Shell.WriteLine("Start kein Update")
                    else
                        labelTarget0:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
                        labelTarget1:SetVisible(true);
                        labelTarget2:SetVisible(true);
                        --Turbine.Shell.WriteLine("Start Update")
                        labelTarget1:SetWantsUpdates(true);
                    end
                else
                    labelTarget0:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
                    labelTarget1:SetVisible(false);
                    labelTarget2:SetVisible(false);
                    labelTarget4:SetVisible(false); -- visible
                    labelTarget5:SetVisible(false); -- visible
                    labelTarget0:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
                    labelTarget1:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
                    labelTarget2:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
                    --labelTarget3:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
                    targetWindow:SetBackColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 0, 0, 0));
                    Turbine.Shell.WriteLine("Power nil")
                end
                labelTarget0:SetText(" " .. targetName);
                targetWindow:SetVisible(true);
            end
    
            test.Update = function()
                local curtime = Turbine.Engine.GetGameTime()
                if (curtime >= endtime) then
                    local targetMorale = math.floor(target:GetMorale())
                    local targetMoraleStr = tostring(targetMorale);
                    labelTarget1:SetText(targetMoraleStr);
                    if (targetMorale ~= 0) then
                        labelTarget0:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
                        labelTarget1:SetVisible(true);
                        labelTarget2:SetVisible(true);
    
                        labelTarget1:SetWantsUpdates(true);                
                    end
                    test:SetWantsUpdates(false);
                end
            end
        end
    end

    labelTarget1.Update = function ()       
        --Turbine.Shell.WriteLine("Updater")
        if target ~= nil then 
            if (target.GetPower~=nil) then
                local targetMorale = math.floor(target:GetMorale())
                local targetMaxMorale = math.floor(target:GetMaxMorale())
                local targetPercentMorale = math.floor(targetMorale / targetMaxMorale * 100);
                local targetMoraleStr = tostring(targetMorale);
                local targetMaxMoraleStr = tostring(targetMaxMorale);
                local targetPercentMoraleStr = tostring(targetPercentMorale);
                if targetPercentMorale == 100 then
                    labelTarget2:SetText("100 % ");
                    -- set label visibility
                    labelTarget4:SetVisible(false); -- visible
                    labelTarget5:SetVisible(false); -- visible
                    -- opaticity change from text labels
                    labelTarget0:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
                    labelTarget1:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
                    labelTarget2:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
                    --labelTarget3:SetForeColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 1, 1, 1));
                    targetWindow:SetBackColor(Turbine.UI.Color(SETTINGS.WINDOWS.OPACITY, 0, 0, 0));
        
                    labelTarget1:SetText(targetMoraleStr);
                    --Turbine.Shell.WriteLine("if")
                elseif targetMorale == 0 then
                    --Turbine.Shell.WriteLine("elseif")
                    labelTarget1:SetWantsUpdates(false);
                else
                    labelTarget1:SetText(targetMoraleStr .. " / " .. targetMaxMoraleStr);
                    -- set label visibility
                    labelTarget4:SetVisible(true); -- visible
                    labelTarget5:SetVisible(true); -- visible
                    -- opaticity change from text labels
                    labelTarget0:SetForeColor(Turbine.UI.Color(1, 1, 1, 1));
                    labelTarget1:SetForeColor(Turbine.UI.Color(1, 1, 1, 1));
                    labelTarget2:SetForeColor(Turbine.UI.Color(1, 1, 1, 1));
                    --labelTarget3:SetForeColor(Turbine.UI.Color(1, 1, 1, 1));
                    targetWindow:SetBackColor(Turbine.UI.Color(0.8, 0, 0, 0));
                    targetWindow:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend );
                            
                    labelTarget2:SetText(targetPercentMoraleStr .. " % ");
    
                    local tarwidthChange = SETTINGS.WINDOWS.TARWIDTH * targetPercentMorale / 100;
                    labelTarget4:SetSize(tarwidthChange, SETTINGS.WINDOWS.TARHEIGHT);
    
                    -- SETTINGS.WINDOWS.WIDTH change for morale between two hits
                    local tarwidthDelta = tarwidthLast - tarwidthChange;
                    tarwidthLast = tarwidthLast - tarwidthDelta;
                    labelTarget5:SetSize((tarwidthLast + tarwidthDelta), SETTINGS.WINDOWS.TARHEIGHT);
                    --Turbine.Shell.WriteLine("else")
                end
            else
                labelTarget1:SetWantsUpdates(false);
                --Turbine.Shell.WriteLine("2")
            end
        else
            labelTarget1:SetWantsUpdates(false);
            --Turbine.Shell.WriteLine("nil")
        end
    end    
end


------------------------------------------------------------------
--option panal


function DrawOptionsControl()
    options = Turbine.UI.Control();
    plugin.GetOptionsPanel = function(self) return options; end

    options:SetBackColor(Turbine.UI.Color(0.1, 0.1, 0.1)); -- RGB, 0 = 0, 1 = 255.
    options:SetSize(320, 520);

    -- add a label for load/unload original vitalbar
    local vitalLabel = Turbine.UI.Label();
    vitalLabel:SetParent(options);
    vitalLabel:SetSize(300, 25);
    vitalLabel:SetPosition(10, 10);
    vitalLabel:SetText("Original Moralebar Switch: On/Off/Auto/AutoInvert");

    local vitalButton = Turbine.UI.Lotro.Button()
    vitalButton:SetParent(options);
    vitalButton:SetSize(30, 15);
    vitalButton:SetPosition(10, 30);
    vitalButton:SetVisible(true);
    vitalButton:SetText("On");
    local k = SETTINGS.WINDOWS.ORIGINAL_MORALEBAR;
    if (k == 0) then
        vitalButton:SetText("Off");
        Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, false );
    elseif (k == 1) then
        vitalButton:SetText("Auto");
        Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, false );
        player.InCombatChanged = function(sender, args)
            if player:IsInCombat() == true then
                Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, true );
            else
                Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, false );
            end            
        end
    elseif (k == 2) then
        vitalButton:SetText("Inv");
        Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, true );
        player.InCombatChanged = function(sender, args)
            if player:IsInCombat() == true then
                Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, false );
            else
                Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, true );
            end            
        end
    else
        vitalButton:SetText("On");
        Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Target, true );
    end
    vitalButton.Click = function(sender, args)
        if k == 0 then
            vitalButton:SetText("Auto");
            Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, false );
            player.InCombatChanged = function(sender, args)
                if player:IsInCombat() == true then
                    Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, true );
                else
                    Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, false );
                end            
            end
            k = k + 1            
            
        elseif k == 1 then
            vitalButton:SetText("Inv");
            Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, true );
            player.InCombatChanged = function(sender, args)
                if player:IsInCombat() == true then
                    Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, false );
                else
                    Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, true );
                end            
            end
            k = k + 1             

          elseif k == 2 then
            Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, true );
            vitalButton:SetText("On");
            k = k + 1
            
        else
            Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, false );
            vitalButton:SetText("Off");
            k = 0;
        end
        SETTINGS.WINDOWS.ORIGINAL_MORALEBAR = k
    end

    -- add a label for load/unload original targetbar
    local targetLabel = Turbine.UI.Label();
    targetLabel:SetParent(options);
    targetLabel:SetSize(200, 25);
    targetLabel:SetPosition(10, 70);
    targetLabel:SetText("Original Targetbar Switch: ");

    local targetButton = Turbine.UI.Lotro.Button()
    targetButton:SetParent(options);
    targetButton:SetSize(30, 15);
    targetButton:SetPosition(10, 90);
    targetButton:SetVisible(true);
    local j = SETTINGS.WINDOWS.ORIGINAL_TARGETBAR;
    if (j == 0) then
        targetButton:SetText("Off");
        Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Target, false );
    else
        targetButton:SetText("On");
        Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Target, true );
    end
    
    targetButton.Click = function(sender, args)
        if (j == 1) then
            Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Target, false );
            targetButton:SetText("Off");
            j = 0
        else
            Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Target, true );
            targetButton:SetText("On");
            j = 1 ;
        end
        SETTINGS.WINDOWS.ORIGINAL_TARGETBAR = j
    end

    -- add a label for load/unload original vitalbar
    local hightfromgroundLabel = Turbine.UI.Label();
    hightfromgroundLabel:SetParent(options);
    hightfromgroundLabel:SetSize(200, 25);
    hightfromgroundLabel:SetPosition(10, 130);
    hightfromgroundLabel:SetText("Set hight from Ground for Morale: ");

    local hightfromgroundScrollBar = Turbine.UI.Lotro.ScrollBar()
    hightfromgroundScrollBar:SetParent(options);
    hightfromgroundScrollBar:SetSize(200, 15);
    hightfromgroundScrollBar:SetPosition(10, 160);
    hightfromgroundScrollBar:SetVisible(true);
    
    hightfromgroundScrollBar.ValueChanged = function(sender, args)
        local value = hightfromgroundScrollBar:GetValue() / 100;        
        window:SetPosition(screenWidth/2 - SETTINGS.WINDOWS.WIDTH/2, screenHeight * value);
        hightfromgroundLabel:SetText("Set hight from Ground for Morale: " .. value);
    end

    -- add a label for load/unload original targetbar
    local hightfromgroundTargetLabel = Turbine.UI.Label();
    hightfromgroundTargetLabel:SetParent(options);
    hightfromgroundTargetLabel:SetSize(200, 25);
    hightfromgroundTargetLabel:SetPosition(10, 190);
    hightfromgroundTargetLabel:SetText("Set hight from Ground for Target: ");

    local hightfromgroundTarScrollBar = Turbine.UI.Lotro.ScrollBar()
    hightfromgroundTarScrollBar:SetParent(options);
    hightfromgroundTarScrollBar:SetSize(200, 15);
    hightfromgroundTarScrollBar:SetPosition(10, 220);
    hightfromgroundTarScrollBar:SetVisible(true);
    
    hightfromgroundTarScrollBar.ValueChanged = function(sender, args)
        local value = hightfromgroundTarScrollBar:GetValue() / 100;        
        targetWindow:SetPosition(screenWidth/2 - SETTINGS.WINDOWS.TARWIDTH/2, screenHeight * value);
        hightfromgroundTargetLabel:SetText("Set hight from Ground for Target: " .. value);
    end

    -- add a label for load/unload plugin targetbar
    local PlugintargetLabel = Turbine.UI.Label();
    PlugintargetLabel:SetParent(options);
    PlugintargetLabel:SetSize(200, 25);
    PlugintargetLabel:SetPosition(10, 260);
    PlugintargetLabel:SetText("Plugin Targetbar Switch: ");

    local PlugintargetButton = Turbine.UI.Lotro.Button()
    PlugintargetButton:SetParent(options);
    PlugintargetButton:SetSize(30, 15);
    PlugintargetButton:SetPosition(10, 280);
    PlugintargetButton:SetVisible(true);
    if (SETTINGS.WINDOWS.PLUGIN_TARGETBAR == 0) then
        PlugintargetButton:SetText("Off");
    else
        PlugintargetButton:SetText("On");
    end
    local i = SETTINGS.WINDOWS.PLUGIN_TARGETBAR;
    if (SETTINGS.WINDOWS.PLUGIN_TARGETBAR == 1) then
        targetUI();
        targetMethods();
       
    end
    PlugintargetButton.Click = function(sender, args)
        if (PlugintargetButton:GetText() == "On") then
            PlugintargetButton:SetText("Off");                        
            i = 0
        else
            PlugintargetButton:SetText("On");
            targetUI();
            targetMethods();                     
            i = 1;
        end
        SETTINGS.WINDOWS.PLUGIN_TARGETBAR = i
    end

    -- add a label for load/unload powerbar
    local PowerbarLabel = Turbine.UI.Label();
    PowerbarLabel:SetParent(options);
    PowerbarLabel:SetSize(200, 25);
    PowerbarLabel:SetPosition(10, 320);
    PowerbarLabel:SetText("Powerbar Switch: ");

    local PowerbarLabel = Turbine.UI.Lotro.Button()
    PowerbarLabel:SetParent(options);
    PowerbarLabel:SetSize(30, 15);
    PowerbarLabel:SetPosition(10, 340);
    PowerbarLabel:SetVisible(true);
    local l = SETTINGS.WINDOWS.POWERBAR;
    if (l == 0) then
        PowerbarLabel:SetText("Off");
    else
        PowerbarLabel:SetText("On");
    end
    
    PowerbarLabel.Click = function(sender, args)
        if (l == 1) then
            PowerbarLabel:SetText("Off");                        
            l = 0
        else
            PowerbarLabel:SetText("On");                            
            l = 1;
        end
        SETTINGS.WINDOWS.POWERBAR = l
        playerUI()
    end

    -- add a label for reset settings
    local ResetLabel = Turbine.UI.Label();
    ResetLabel:SetParent(options);
    ResetLabel:SetSize(200, 25);
    ResetLabel:SetPosition(10, 380);
    ResetLabel:SetText("Reset Settings: ");

    local ResetLabel = Turbine.UI.Lotro.Button()
    ResetLabel:SetParent(options);
    ResetLabel:SetSize(30, 15);
    ResetLabel:SetPosition(10, 400);
    ResetLabel:SetVisible(true);
    ResetLabel:SetText("Reset");
    
    ResetLabel.Click = function(sender, args)
        SETTINGS = DEFAULT_SETTINGS
        playerUI()
        targetUI()
        Turbine.Shell.WriteLine("reset Settings.")
    end

    -- add a label for save settings
    local SaveLabel = Turbine.UI.Label();
    SaveLabel:SetParent(options);
    SaveLabel:SetSize(300, 25);
    SaveLabel:SetPosition(10, 480);
    SaveLabel:SetText("Settings will be saved automatically.");
end


function Main()
    LoadSettings();
    RegisterForUnload();
    playerUI();
    playerMethods();
    DrawOptionsControl();    
end

Main();
